"use strict";
/**
*   succ :  {ok: true, data: {...}, ... }
*
*   error:  {error: true, error_msg: "...", error_code: int, 'error_trace': '...", ...}
*   error_trace will no be displayed in prod environment.
*/
var dataCache = {};

var cube = require('./cube');
var URI = require('./uri');

var Request = function(url, uniqueId, method) {

  if (!uniqueId) uniqueId = 0;
  if (!method) method = 'POST';

  this._uniqueKey = url + "_" + uniqueId;
  this.url = url;
  this.method = method;
  this.data = {};
  this.xhr = null;
};

cube.mix(Request, {

  lockObj: {},

  formatUrl: function(url) {
    return url;
  },

  create: function(url, uniqueId, method) {
    url = this.formatUrl(url);

    // check lock
    if (!uniqueId) uniqueId = 0;
    var key = url + "_" + uniqueId;
    if (this.lockObj[key]) {
      return false;
    }

    // lock
    this.lockObj[key] = 1
    var request = new Request(url, uniqueId, method);
    return request;
  }
});

cube.mix(Request.prototype, {

  _loadingElement: null,

  setLoadingElement: function(element) {
    this._loadingElement = element;
  },

  send: function(prepare) {
    var cacheData = dataCache[this._uniqueKey];
    if (cacheData) {
      this.dispatchResponse(cacheData);
      // unlock
      this.unlockRequest();
      return this;
    }
    else {
      var me = this;
      var setting = {
        type: this.method,
        url: this.url,
        data: this.data,
        dataType: 'json'
      };
      if (prepare) {
        setting = prepare(setting);
      }
      this.xhr = $.ajax(setting).success(function(ret) {
        if (!ret.error || ret.ok) {
          me.invokeResponseHandler(ret);
        } else {
          me.invokeErrorHandler(ret);
        }
      }).error(function(ret) {
        me.invokeErrorHandler(ret);
      }).always(function() {
        // unlock
        me.unlockRequest();
      });
      return this;
    }
  },

  unlockRequest: function() {
    delete Request.lockObj[this._uniqueKey];
  },

  setMethod: function(method) {
    this.method = method;
    return this;
  },

  setData: function(data) {
    if (data instanceof FormData) {
      data.append('forajax', 1);
    } else {
      data['forajax'] = 1;
    }
    this.data = data;
    return this;
  },

  setHandler: function(handler) {
    if (cube.isFunction(handler)) {
      this.handler = handler;
    }
    return this;
  },

  getHandler: function() {
    return this.handler;
  },

  setErrorHandler: function(errorHandler) {
    this.errorHandler = errorHandler;
    return this;
  },

  setRelativeTo: function(element) {
    this.relativeTo = element;
    return this;
  },

  getRelativeTo: function() {
    return this.relativeTo;
  },

  abort: function() {
    this.xhr && this.xhr.abort();
    return this;
  },

  invokeResponseHandler: function(ret) {
    if ('resource' in ret) {
      this.loadResource(ret.resource, cube.bind(function() {
        this.dispatchResponse(ret);
      }, this));
    } else {
      this.dispatchResponse(ret);
    }
  },

  invokeErrorHandler: function(ret) {
    this.dispatchErrorResponse(ret);
  },

  loadResource: function(resource, callback) {
    if ('depends' in resource && !cube.isEmpty(resource.depends)) {
      cube.Resource.addResourceDepends(resource.depends);
    }
    if ('resourceMap' in resource && !cube.isEmpty(resource.resourceMap)) {
      cube.Resource.addResourceMapViaKxVersion(resource.resourceMap);
    }
    if ('css' in resource && !cube.isEmpty(resource.css)) {
      cube.Resource.loadCSS(resource.css, callback);
    } else {
      callback();
    }
    if ('js' in resource && !cube.isEmpty(resource.js)) {
      cube.Resource.loadJS(resource.js);
    }
    var me = this;
    if ('onloads' in resource) {
      setTimeout(function() {
        cube.forEach(resource.onloads, function(onload) {
          (new Function(onload)).apply(me);
        });
      });
    }
  },

  dispatchResponse: function(ret) {

    // TOOD: cache by time
    if (ret.cache_time) {
      dataCache[this._uniqueKey] = ret;
    }

    // pop dialog
    if (ret && ret.data && ret.data.pop_dialog) {
      var dialog = ret.data.pop_dialog;

      var msg = dialog.msg;
      var type = dialog.type;

      var data = {auto_close: dialog.auto_close};
      var MsgBox = require('./view/msg-box');
      if (type == 'succ') {
        MsgBox.success(msg, data);
      } else {
        MsgBox.error(msg, data);
      }

      if (dialog.block_handler) {
        return;
      }
    }

    if (this.handler) {
      this.handler(ret);
    }
  },

  dispatchErrorResponse: function(ret) {

    if (this.errorHandler) {
      var processed = this.errorHandler(ret);
      if (processed)
        return;
    }

    var msg = '';
    var width = 650;

    // php exception
    if (ret.error_msg && ret.error) {
      msg = ret.error_msg;
      if (ret.error_trace) {
        msg += ret.error_trace;
      }
      this.showError(msg);
    } else {

      width = 1024;
      msg += ret.status + "<br/>";
      msg += ret.statusText + "<br/>";
      msg += ret.responseText;
      this.showError(msg);
    }
  },

  showError: function(msg) {
    var data = {};
    if (msg.length > 1000) {
      data.width = 960;
    }
    var MsgBox = require('./view/msg-box');
    MsgBox.error(msg, data);
  },
});

cube.Request = Request;
module.exports = Request;
